-- ****************************************************************************
-- Company:         CAEN SpA - Viareggio - Italy
-- Model:           V1495 -  Multipurpose Programmable Trigger Unit
-- FPGA Proj. Name: V1495USR_DEMO
-- Device:          ALTERA EP1C4F400C6
-- Author:          Luca Colombini
-- Date:            02-03-2006
-- ----------------------------------------------------------------------------
-- Module:          V1495USR_PKG
-- Description:     Package that implements global constants (ID Codes, 
--                  Register Addresses).
-- ****************************************************************************

-- ############################################################################
-- Revision History:
-- ############################################################################
LIBRARY ieee;
USE ieee.std_logic_1164.all;
PACKAGE v1495pkg IS
-- Custom Types
type logic_vector32_array is array(integer range <>) of std_logic_vector(31 downto 0);
type logic_vector16_array is array(integer range <>) of std_logic_vector(15 downto 0);
type logic_vector8_array  is array(integer range <>) of std_logic_vector( 7 downto 0);
   
-- Constants
-- Expansion Mezzanine Type ID-Codes
constant A395A : std_logic_vector(2 downto 0) := "000"; -- 32CH IN LVDS/ECL INTERFACE
constant A395B : std_logic_vector(2 downto 0) := "001"; -- 32CH OUT LVDS INTERFACE
constant A395C : std_logic_vector(2 downto 0) := "010"; -- 32CH OUT ECL INTERFACE
constant A395D : std_logic_vector(2 downto 0) := "011"; -- 8CH I/O SELECT NIM/TTL INTER

-- Register Address Map
constant A_ASTATUS_L  : std_logic_vector(15 downto 0) := X"1000";
constant A_ASTATUS_H  : std_logic_vector(15 downto 0) := X"1002";
constant A_BSTATUS_L  : std_logic_vector(15 downto 0) := X"1004";
constant A_BSTATUS_H  : std_logic_vector(15 downto 0) := X"1006";
constant A_CSTATUS_L  : std_logic_vector(15 downto 0) := X"1008";
constant A_CSTATUS_H  : std_logic_vector(15 downto 0) := X"100A";
constant A_AMASK_L    : std_logic_vector(15 downto 0) := X"100C";
constant A_AMASK_H    : std_logic_vector(15 downto 0) := X"100E";
constant A_BMASK_L    : std_logic_vector(15 downto 0) := X"1010";
constant A_BMASK_H    : std_logic_vector(15 downto 0) := X"1012";
constant A_CMASK_L    : std_logic_vector(15 downto 0) := X"1014";
constant A_CMASK_H    : std_logic_vector(15 downto 0) := X"1016";
constant A_GATEWIDTH  : std_logic_vector(15 downto 0) := X"1018";
constant A_CCTRL_L    : std_logic_vector(15 downto 0) := X"101A";
constant A_CCTRL_H    : std_logic_vector(15 downto 0) := X"101C";
constant A_MODE       : std_logic_vector(15 downto 0) := X"101E";
constant A_SCRATCH    : std_logic_vector(15 downto 0) := X"1020";
constant A_GCTRL      : std_logic_vector(15 downto 0) := X"1022";
constant A_DCTRL_L    : std_logic_vector(15 downto 0) := X"1024";
constant A_DCTRL_H    : std_logic_vector(15 downto 0) := X"1026";
constant A_DDATA_L    : std_logic_vector(15 downto 0) := X"1028";
constant A_DDATA_H    : std_logic_vector(15 downto 0) := X"102A";
constant A_ECTRL_L    : std_logic_vector(15 downto 0) := X"102C";
constant A_ECTRL_H    : std_logic_vector(15 downto 0) := X"102E";
constant A_EDATA_L    : std_logic_vector(15 downto 0) := X"1030";
constant A_EDATA_H    : std_logic_vector(15 downto 0) := X"1032";
constant A_FCTRL_L    : std_logic_vector(15 downto 0) := X"1034";
constant A_FCTRL_H    : std_logic_vector(15 downto 0) := X"1036";
constant A_FDATA_L    : std_logic_vector(15 downto 0) := X"1038";
constant A_FDATA_H    : std_logic_vector(15 downto 0) := X"103A";
constant A_REVISION   : std_logic_vector(15 downto 0) := X"103C";
constant A_PDL_CTRL   : std_logic_vector(15 downto 0) := X"103E";
constant A_PDL_DATA   : std_logic_vector(15 downto 0) := X"1040";
constant A_DIDCODE    : std_logic_vector(15 downto 0) := X"1042";
constant A_EIDCODE    : std_logic_vector(15 downto 0) := X"1044";
constant A_FIDCODE    : std_logic_vector(15 downto 0) := X"1046";
-- Custom Register Addresses
constant A_CREVISION      : std_logic_vector(15 downto 0) := X"1100";
constant A_CSCRATCH       : std_logic_vector(15 downto 0) := X"1102";
constant A_CUSTOM_CNTRL   : std_logic_vector(15 downto 0) := X"1104";
constant A_CUSTOM_STATUS  : std_logic_vector(15 downto 0) := X"1106";
constant A_COINC_RESET    : std_logic_vector(15 downto 0) := X"1108";
constant A_SIGATEWIDTH    : std_logic_vector(15 downto 0) := X"110A";
constant A_SIRESDELAY     : std_logic_vector(15 downto 0) := X"110C";
constant A_SIDELAY_0      : std_logic_vector(15 downto 0) := X"1300";
constant A_SIDELAY_1      : std_logic_vector(15 downto 0) := X"1302";
constant A_SIDELAY_2      : std_logic_vector(15 downto 0) := X"1304";
constant A_SIDELAY_3      : std_logic_vector(15 downto 0) := X"1306";
constant A_BGODELAY_SHORT : std_logic_vector(15 downto 0) := X"1308";
constant A_BGODELAY_LONG  : std_logic_vector(15 downto 0) := X"1310";
constant A_BGOBLOCKDELAY  : std_logic_vector(15 downto 0) := X"1312";
constant A_ATRIGREG_L     : std_logic_vector(15 downto 0) := X"1110";
constant A_ATRIGREG_H     : std_logic_vector(15 downto 0) := X"1112";
constant A_BTRIGREG_L     : std_logic_vector(15 downto 0) := X"1114";
constant A_BTRIGREG_H     : std_logic_vector(15 downto 0) := X"1116";
constant A_COINCREG_0     : std_logic_vector(15 downto 0) := X"1120";
constant A_COINCMASK_0    : std_logic_vector(15 downto 0) := X"1160";
constant A_COUNTERREG_0   : std_logic_vector(15 downto 0) := X"11F8";
constant A_COUNTERREG_1   : std_logic_vector(15 downto 0) := X"11FA";
constant A_COUNTERREG_2   : std_logic_vector(15 downto 0) := X"11FC";
constant A_COUNTERREG_3   : std_logic_vector(15 downto 0) := X"11FE";
constant A_ATIME_L : logic_vector16_array(0 to 31) := (X"1200",X"1204",X"1208",X"120C",
                                                       X"1210",X"1214",X"1218",X"121C",
                                                       X"1220",X"1224",X"1228",X"122C",
                                                       X"1230",X"1234",X"1238",X"123C",
                                                       X"1240",X"1244",X"1248",X"124C",
                                                       X"1250",X"1254",X"1258",X"125C",
                                                       X"1260",X"1264",X"1268",X"126C",
                                                       X"1270",X"1274",X"1278",X"127C");
constant A_ATIME_H : logic_vector16_array(0 to 31) := (X"1202",X"1206",X"120A",X"120E",
                                                       X"1212",X"1216",X"121A",X"121E",
                                                       X"1222",X"1226",X"122A",X"122E",
                                                       X"1232",X"1236",X"123A",X"123E",
                                                       X"1242",X"1246",X"124A",X"124E",
                                                       X"1252",X"1256",X"125A",X"125E",
                                                       X"1262",X"1266",X"126A",X"126E",
                                                       X"1272",X"1276",X"127A",X"127E");
constant A_BTIME_L : logic_vector16_array(0 to 31) := (X"1280",X"1284",X"1288",X"128C",
                                                       X"1290",X"1294",X"1298",X"129C",
                                                       X"12A0",X"12A4",X"12A8",X"12AC",
                                                       X"12B0",X"12B4",X"12B8",X"12BC",
                                                       X"12C0",X"12C4",X"12C8",X"12CC",
                                                       X"12D0",X"12D4",X"12D8",X"12DC",
                                                       X"12E0",X"12E4",X"12E8",X"12EC",
                                                       X"12F0",X"12F4",X"12F8",X"12FC");
constant A_BTIME_H : logic_vector16_array(0 to 31) := (X"1282",X"1286",X"128A",X"128E",
                                                       X"1292",X"1296",X"129A",X"129E",
                                                       X"12A2",X"12A6",X"12AA",X"12AE",
                                                       X"12B2",X"12B6",X"12BA",X"12BE",
                                                       X"12C2",X"12C6",X"12CA",X"12CE",
                                                       X"12D2",X"12D6",X"12DA",X"12DE",
                                                       X"12E2",X"12E6",X"12EA",X"12EE",
                                                       X"12F2",X"12F6",X"12FA",X"12FE");

END v1495pkg;
